from dataclasses import dataclass, field
from typing import List

from fuel_efficency.algorithms.dijkstra import DijkstraStrategy
from fuel_efficency.algorithms.path_finding import PathfindingStrategy
from fuel_efficency.entities.node import Node
from fuel_efficency.entities.valley import Valley


@dataclass(slots=True)
class Context:
    _strategy: PathfindingStrategy = field(default_factory=DijkstraStrategy)
    _grid: List[List[Node]] = field(default_factory=lambda: [[Valley() for _ in range(3)] for _ in range(3)])
    _start: Node = field(default_factory=Valley)
    _end: Node = field(default_factory=Valley)

    @property
    def grid(self):
        return self._grid

    @grid.setter
    def grid(self, new_grid: List[List[Node]]):
        if not isinstance(new_grid, list):
            raise TypeError("Grid must be a list")
        if not all(isinstance(row, list) for row in new_grid):
            raise TypeError("Grid must be a list of lists")
        self._grid = new_grid

    @property
    def start(self):
        return self._start

    @start.setter
    def start(self, new_start: Node):
        self._start = new_start

    @property
    def end(self):
        return self._end

    @end.setter
    def end(self, new_end: Node):
        self._end = new_end

    @property
    def strategy(self):
        return self._strategy

    @strategy.setter
    def strategy(self, new_strategy: PathfindingStrategy):
        if not isinstance(new_strategy, PathfindingStrategy):
            raise TypeError("Strategy must be an instance of PathfindingStrategy")
        self._strategy = new_strategy

    def run(self):
        if not hasattr(self._strategy, 'find_path'):
            raise NotImplementedError("Strategy must implement the find_path method")
        return self._strategy.find_path(self.grid, self.start, self.end)



from dataclasses import dataclass

from fuel_efficency.entities.node import Node
from fuel_efficency.entities.position import Position


@dataclass(slots=True)
class Valley:
   weight: float = float(1)
   position: 'Position' = Position()



import sys
from dataclasses import dataclass
from typing import Optional


@dataclass(slots=True)
class Position:
    x: int = sys.maxsize
    y: int = sys.maxsize

    def __add__(self, other:'Position') -> Optional['Position']:
        """
        Add two Position objects together.

        Args:
            other (Position): The other Position object to add to this one.

        Returns:
            Position: The sum of the two Position objects.
        """
        pass

    def __sub__(self, other:'Position') -> Optional['Position']:
        """
        Subtract two Position objects.

        Args:
            other (Position): The other Position object to subtract from this one.

        Returns:
            Position: The difference of the two Position objects.
        """
        pass



import heapq
import math
from typing import List

from fuel_efficency.algorithms.path_finding import PathfindingStrategy
from fuel_efficency.entities.node import Node
from fuel_efficency.entities.position import Position


class AStarStrategy(PathfindingStrategy):

    allowed_directions = [Position(-1, 0), Position(0, -1), Position(0, 1), Position(1, 0)]



from dataclasses import dataclass

from fuel_efficency.entities.node import Node
from fuel_efficency.entities.position import Position


@dataclass(slots=True)
class UpHill:
    weight: float = float(2)
    position: 'Position' = Position()



from dataclasses import dataclass
from typing import Protocol

from fuel_efficency.entities.position import Position


@dataclass(slots=True)
class Node(Protocol):
    weight: float
    position: 'Position' = Position()



from setuptools import setup, find_packages

setup(
    name='bees-challenge',
    version='0.9.0',
    author='yuiti',
    author_email='yuiti.usp@gmail.com',
    description='challenge submission for the bees ml path challenge',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/yuiti-ara/yuiti-bees-ml-path-challenge',
    packages=find_packages(),
    python_requires='>=3.10',
    install_requires=[],
    entry_points={
        'console_scripts': [],
    },
    include_package_data=False,
    zip_safe=True,
)
